/*
 * HTTP Client
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <stdio.h>
#include <errno.h>
#include <memory.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/times.h>

enum HTTP_METHOD
{
    HTTP_GET,
    HTTP_POST
};

typedef unsigned short WORD;

int connect_socket(const char *hostname, WORD portnum)
{
    struct sockaddr_in sa;
    struct hostent     *hp;
    int s;
    
    if ((hp = gethostbyname(hostname)) == NULL) { /* do we know the host's */
    errno = ECONNREFUSED;                       /* address? */
    return(-1);                                /* no */
    }
    
    memset(&sa,0,sizeof(sa));
    memcpy((char *)&sa.sin_addr, hp->h_addr, hp->h_length);     /* set address */
    sa.sin_family = hp->h_addrtype;
    sa.sin_port = htons((u_short)portnum);
    
    if ((s = socket(hp->h_addrtype,SOCK_STREAM,0)) < 0)     /* get socket */
    return(-1);
    if (connect(s, (struct sockaddr *)&sa, sizeof sa) < 0) { /* connect */
        close(s);
        return(-1);
    }
    return s;
}

/*
 * Note: I have increased all strncpy to more than it actually needs because
 * if a person changes "GET " to "GET /rs/blah.php?a=ddd", they will forget to 
 * change the number. They might also forget to change meth_size, but that's 
 * their problem. I can only help them so much. This helps significantly for 
 * variable length. Content-Length:%i. Guten tag!
 */
int send_request(char *hostname, int sock, char *filename, enum HTTP_METHOD method, char *post_data, int post_length)
{
    char buf[10*1024];
    int meth_size;
    if(method == HTTP_GET)
    {
        strncpy(buf, "GET ", 1024);
        meth_size = 4;
    }
    else
    {
        strncpy(buf, "POST ", 1024);
        meth_size = 5;
    }
    // Validate filename to ensure legitimacy.
    int file_len = strlen(filename);
    if(file_len > 1024) return -1;
    strncpy(buf+meth_size, filename, 1025);
    strncpy(buf+meth_size+file_len, " HTTP/1.1\nHost:", 1024);
    // Validate hostname to ensure legitimacy.
    int host_len = strlen(hostname);
    if(host_len > 1024) return -1;
    strncpy(buf+meth_size+file_len+15, hostname, 1025);
    
    int other_len;
    if(post_length > 0)
    {
        // TODO: Setup the post data.
        strncpy(buf+meth_size+file_len+15+host_len, "\nContent-Length:12345\n\n", 1024);
        char len_string[6];
        snprintf(len_string, 6, "%05i", post_length);
        buf[meth_size+file_len+15+host_len+15] = len_string[0];
        buf[meth_size+file_len+15+host_len+16] = len_string[1];
        buf[meth_size+file_len+15+host_len+17] = len_string[2];
        buf[meth_size+file_len+15+host_len+18] = len_string[3];
        buf[meth_size+file_len+15+host_len+19] = len_string[4];
        memcpy(buf+meth_size+file_len+15+host_len+23, post_data, post_length);
        other_len = 23 + post_length;
    }
    else
    {
        strncpy(buf+meth_size+file_len+15+host_len, "\n\n", 3);
        other_len = 3;
    }
    
    int buf_size = meth_size + file_len + 15 + host_len + 3;
    int rc;
    
    if ((rc = write(sock, buf, buf_size)) < 0)
    {
        return -1;
    }
    return 0;
}

int hexbtoi(char hex)
{
    if(hex >= '0' && hex <= '9')
    {
        return hex - '0';
    }
    if(hex >= 'a' && hex <= 'f')
    {
        return hex - 'a' + 10;
    }
    if(hex >= 'A' && hex <= 'F')
    {
        return hex - 'A' + 10;
    }
    return 0;
}

int hextoi(char *buf, int len)
{
    int i = 0;
    while(buf[i] != '\n' && i < len) { i++; }
    //printf("hex: ");
    //fwrite(buf, i, 1, stdout);
    //printf("\n");
    int value = i+3;
    i -= 2;
    int modifier = 1;
    while(i >= 0)
    {
        //printf("value += %i * %i\n", hexbtoi(buf[i]), modifier);
        value += hexbtoi(buf[i]) * modifier;
        modifier *= 16;
        i--;
    }
    //printf("value = %i\n", value);
    return value;
}

int read_header(char *buf, int amt_read, int *body_pos)
{
    int header_size = -1;
    int chunked = 0;
    int i = 0;
    
    if(strncmp("HTTP/1.1 200 OK", buf, 1024) == 0)
    {
        // Found header 1, skip to next \n.
        while(buf[i] != '\n' && i < amt_read) { i++; }
    }
    else if(strncmp("HTTP/1.1 302 Found", buf, 1024) == 0)
    {
        while(buf[i] != '\n' && i < amt_read) { i++; }
    }
    else
    {
        /*printf("HTTP/1.1 200 OK needed, but not found.\n");
        fwrite(buf, amt_read, 1, stdout);
        printf("\n");*/
        while(buf[i] != '\n' && i < amt_read) { i++; }
        //return -2;
    }
    for(i = 0; i < amt_read; i++)
    {
        // Found the start of the body, hooray!
        if(buf[i] == '\r' && buf[i + 1] == '\n')
        {
            *body_pos = i + 2;
            if(chunked == 0)
            {
                return header_size;
            }
            else
            {
                i += 2;
                break;
            }
        }
        else if(strncmp("Content-Length:", buf + i, 1024) == 0)
        {
            // Found length header, set header_size;
            header_size = atoi(buf+i+15);
        }
        else if(strncmp("Transfer-Encoding: chunked", buf + i, 1024) == 0)
        {
            // If it is chunked, the body will go like this:
            // 10d
            // 10d worth of stuff.
            // another hex encoded int.
            // ... worth of stuff.
            // repeat many times.
            // 0
            // end of stream.
            chunked = 1;
        }
        else
        {
            while(buf[i] != '\n' && i < amt_read) { i++; }
        }
    }
    if(chunked)
    {
        header_size = 0;
        // read the chunked data to find the header size.
        while(i < amt_read)
        {
            int chunk = hextoi(buf+i, amt_read-i);
            header_size += chunk;
            if(chunk == 5) return header_size;
            i += chunk;
        }
    }
    // return -1 if you do not find the end.
    return -1;
}

int recv_response(int sock, char *buf, int buf_size)
{
    char *bpt;
    int count = buf_size;
    int amt_read = 0;
    int body_pos = -1;
    int header_size = -1;
    bpt = buf;
    
    while(amt_read < buf_size)
    {
        //printf("read %i/%i\n", amtRead, buf_size);
        //fwrite(buf, amtRead, 1, stdout);
        //printf("\n");
        fflush(stdout);
        int rc;
        if((rc = read(sock, bpt, count)) <= 0)
        {
            return rc;
        }
        amt_read += rc;
        count -= rc;
        bpt += rc;
        if(header_size < 0)
        {
            header_size = read_header(buf, amt_read, &body_pos);
        }
        if(header_size >= 0)
        {
            //printf("%i, %i, %i, %i\n", amt_read, header_size, body_pos, (100 * amt_read / (header_size + body_pos)));
            if(amt_read >= header_size + body_pos)
            {
                return 0;
            }
            else
            {
                // Progress can be put here:
                // pct = 100 * (amt_read / header_size + body_pos)
            }
        }
    }
    return 0;
}

#ifdef TEST
int main(int argc, char **argv)
{
    char altsci[] = "www.altsci.com";
    char index[] = "/";
    char *server_name = altsci;
    char *filename = index;
    WORD port = 80;
    
    if(argc > 1)
    {
        server_name = argv[1];
    }
    if(argc > 2)
    {
        filename = argv[2];
    }
    
    int s = connect_socket(server_name, port);
    if(s < 0)
    {
        perror("client: connect");
        return 1;
    }
    int ret = send_request(server_name, s, filename, HTTP_GET, "", 0);
    if(ret < 0)
    {
        close(s);
        perror("client: writing on socket stream");
        return 1;
    }
    char buf[10*1024];
    ret = recv_response(s, buf, 10*1024);
    if(ret < 0)
    {
        close(s);
        perror("client: reading socket stream");
        return 1;
    }
    buf[10*1024] = 0;
    fwrite(buf, strlen(buf), 1, stdout);
    close(s);
    printf("\n");
    return 0;
}
#endif
