#ifndef _LIST_UTILS_H_

#define _LIST_UTILS_H_

#include <time.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#include "ssl_bf/ssl_bf2a.h"

struct recv_packet_list;

typedef struct recv_packet_list {
	BYTE packet[1024+52];
	int packet_size;
	int32_t packet_number;
	time_t arrival_time;
	struct sockaddr_in oaddr;
	struct recv_packet_list *next, *prev;
} recv_packet_list;

void recv_packet_list_init(recv_packet_list *recv, BYTE *packet, int packet_size, int32_t packet_number, struct sockaddr_in *oaddr);
void recv_packet_list_add(recv_packet_list *recv, recv_packet_list *child);
void recv_packet_list_add_value(recv_packet_list *recv, BYTE *packet, int packet_size, int32_t packet_number, struct sockaddr_in *oaddr);
recv_packet_list *recv_packet_list_search(const recv_packet_list *recv, int32_t packet_mask, int position);
void recv_packet_list_remove(recv_packet_list *recv);
int recv_packet_list_length(const recv_packet_list *recv);
int recv_packet_list_nlength(const recv_packet_list *recv, int count);
void recv_packet_list_print(FILE *out_file, const recv_packet_list *recv);
void recv_packet_list_destroy(recv_packet_list *recv);

struct recv_list;
// 8 bytes
typedef struct recv_list {
	int32_t packet_number;
	BYTE packet[1024];
	int packet_size;
	struct recv_list *next_recv; // Single linked list.
} recv_list;

#define MEMORY_TEST 0
#ifndef MEMORY_TEST
#define MEMORY_TEST 1
#endif // MEMORY_TEST

void recv_list_init(recv_list *recv, int32_t packet_number, BYTE *data, int data_size);
void recv_list_add(recv_list *recv, recv_list *child);
void recv_list_add_value(recv_list *recv, int32_t packet_number, BYTE *data, int data_size);
void recv_list_check(const recv_list *recv, int32_t *min_out, int32_t *max_out, recv_list *missing);
int recv_list_length(const recv_list *recv);
int recv_list_nlength(const recv_list *recv, int count);
void recv_list_print(FILE *out_file, const recv_list *recv);
void recv_list_write(const recv_list *recv, FILE *out_file);
void recv_list_nwrite(const recv_list *recv, FILE *out_file, int32_t i);
void recv_list_destroy(recv_list *recv);



#endif // _LIST_UTILS_H_
