#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <signal.h>
#include <math.h>
#include <string.h>

#include <pthread.h>

#include "list_utils.h"

typedef struct recv_thread_input {
	int s;
	struct sockaddr_in saddr;
	recv_packet_list packets;
} recv_thread_input;

void *test_thread(void *arg)
{
	int incoming_data = (int)arg;
	printf("incoming_port: %i\n", incoming_data);
	return 0;
}
	
void *recv_thread(void *arg)
{
	recv_thread_input *conn = (recv_thread_input *)arg;
	int i = 0;
	while(1)
	{
		int recv_size;
		char out[1024+52];
		struct sockaddr_in oaddr;
		socklen_t addr_size = sizeof(struct sockaddr_in);
		recv_size = recvfrom(conn->s, out, 1024+52, 0, (struct sockaddr *)&oaddr, &addr_size);
		out[recv_size] = 0;
		if(memcmp(out, "/quit", 5) == 0)
			break;
		printf("recvfrom(%i): %s\n", recv_size, out);
		recv_packet_list_add_value(&(conn->packets), out, recv_size, i);
		i++;
	}
	recv_packet_list_print(&(conn->packets));
	recv_packet_list_destroy(&(conn->packets));
	return 0;
}

int err1(int i, const char *data)
{
	printf("Error: %s\n", data);
	return 0;
}

int main(int argc, char **argv)
{
	
	int local_io_port = 1234;
	char local_io_addr[] = "0.0.0.0";
	
	pthread_t recv_tid;
	pthread_attr_t recv_attr;

	recv_thread_input conn;
	conn.s = socket(PF_INET, SOCK_DGRAM, 0);
	if(conn.s < 0) err1(1, "socket: SOCK_DGRAM");

	memset(&conn.saddr, 0, sizeof(struct sockaddr_in));
	conn.saddr.sin_family = AF_INET;
	conn.saddr.sin_port = htons(local_io_port);
	conn.saddr.sin_addr.s_addr = inet_addr(local_io_addr);
	
	//fprintf(stderr, "bind(%s, %i)\n", local_io_addr, local_io_port);
	int ret = bind(conn.s, (struct sockaddr *)&conn.saddr, sizeof(struct sockaddr_in));
	if(ret) err1(1, "bind");
	
	recv_packet_list_init(&(conn.packets), "", 0, -1);
	
	pthread_attr_init(&recv_attr);
	pthread_attr_setdetachstate(&recv_attr,PTHREAD_CREATE_DETACHED);
	pthread_attr_setscope(&recv_attr, PTHREAD_SCOPE_SYSTEM);
	pthread_create(&recv_tid, &recv_attr, recv_thread, (void *)&conn);
	
	while(1)
	{
		// this thread is not very cool because it finishes quickly.
		// Good threads last until they have done the work of 1000 functions.
		pthread_t tid;
		pthread_attr_t attr;
		pthread_attr_init(&attr);
		pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_DETACHED);
		pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
		pthread_create(&tid, &attr, test_thread, (void *)local_io_port);
		
		char buf[1024];
		fread(buf, 1024, 1, stdin);
	}
	recv_packet_list_destroy(&(conn.packets));
	return 0;
}

