
#ifndef _UDP_SESS1_H_
#define _UDP_SESS1_H_

/*
#ifdef _DEBUG
#define VERBOSE 1
#else
#define VERBOSE 0
#endif
*/

#define nsec 1
#define default_tries 100

typedef enum UDP_CONTROL
{
    UDP_UNKNOWN = 0,
    UDP_QUIT = 1,
    UDP_SEND = 2,
    UDP_SENDTO = 3,
    UDP_RECV = 4,
    UDP_RECVFROM = 5,
    UDP_CONNECT = 6,
    UDP_ERROR = 7,
	UDP_POKE = 8,
    UDP_TIMEOUT = 9
} UDP_CONTROL;

typedef enum UDP_AUTH
{
    UDP_AUTH_UNKNOWN = 0,
    UDP_AUTH_NULL = 1,
    UDP_AUTH_PASSWORD = 2,
    UDP_AUTH_DSA = 3,
} UDP_AUTH;

typedef struct recv_thread_input {
	int s;
	struct sockaddr_in saddr;
	recv_packet_list packets;
	pthread_mutex_t mut;
	BYTE secret[256];
	int secret_len;
	int close;
	int tspeed;
	int verbosity;
	FILE *info_file;
	FILE *err_file;
} recv_thread_input;

static sigjmp_buf jmpbuf2;
static int canjump2;

const char *udp_control_str(UDP_CONTROL c);
static void sigint_return(int signo);
UDP_CONTROL send_data(char *data, int *data_size, int s, struct sockaddr *raddr);
void *recv_thread(void *arg);
UDP_CONTROL recv_timeout1(recv_thread_input *conn, recv_packet_list **found, int32_t packet_mask);
UDP_CONTROL recv_timeout1_quick(char *out, int *recv_size, recv_thread_input *conn, struct sockaddr_in *raddr, int32_t packet_mask);
void use_packet(char *out, int *recv_size, recv_thread_input *conn, recv_packet_list *found, struct sockaddr_in *oaddr);
UDP_CONTROL send_poke(struct sockaddr_in *raddr, recv_thread_input *conn, int tries);
BIGNUM *recv_dh_public_key(recv_thread_input *conn);
void send_public_key(BIGNUM *pub_key, int s, struct sockaddr_in *raddr);
int dh_keyex(struct sockaddr_in *raddr, struct sockaddr_in *oaddr, recv_thread_input *conn, int tries);
int authenticate(const char *password, struct sockaddr_in *raddr, struct sockaddr_in *oaddr, UDP_AUTH auth, recv_thread_input *conn, int tries);

int chat_main(recv_thread_input *conn, struct sockaddr_in *saddr, struct sockaddr_in *raddr);
int send_main(recv_thread_input *conn, FILE *data_io, struct sockaddr_in *saddr, struct sockaddr_in *raddr);
int recv_main(recv_thread_input *conn, FILE *data_io, struct sockaddr_in *saddr, struct sockaddr_in *raddr);
int auth_system(recv_thread_input *conn, struct sockaddr_in *raddr, UDP_AUTH auth, int tries);

int print_stats(recv_thread_input *conn, int32_t last_packet_number, int32_t packets, const recv_list *recv_numbers, time_t start_time, char end_line);

#endif // _UDP_SESS1_H_
